"""."""

import pytest

from nuscenes.utils.data_classes import Box

from association_studio.score_driver.score_driver import ScoreDriver


def test_compute_scores(box: Box, driver: ScoreDriver) -> None:
    driver.compute_scores(box)
    ref = [0.13737373737373737, 0.1866537467790667, 0.7054992044360596, 0.5343434343434343]
    assert driver.scores == pytest.approx(ref)
